package GUI;

import java.awt.EventQueue;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import Concordance.FileInputReader;

/**
 * 
 * This class provides the entire Graphical User Interface for the Concordance Program.
 * It consists of swing components allowing the user to interact and gain more information about their chosen index word.
 * 
 * 
 * @author Jonathan Field jof2@aber.ac.uk 073717922
 * @version 1.00
 * 
 * 
 *
 */


public class ConcordanceFrame extends JFrame {

	private JPanel contentPane;
	private JList lineNumberList;
	private FileInputReader fileReader = new FileInputReader();
	private JTextField txtFieldSelectedWord;
	private JList indexWordList;
	private JPanel panel;
	private String indexToLoad;
	private CustomModelLineNumbers customLineNumberModel;
	private JTextArea txtFieldContext;
	private String textToLoad;

	/**
	 * Launch the application.
	 */
	public void startGUI() {
		EventQueue.invokeLater(new Runnable() {
			@Override
			public void run() {
				try {

					ConcordanceFrame frame = new ConcordanceFrame();
					frame.setVisible(true);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Creates all the Swing components and adds them to the Frame.
	 * The constructor also provides methods for dealing with item changed state listeners to dynamically generate the next JList
	 */
	public ConcordanceFrame() {

		
		try {
		 for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
			        if ("Nimbus".equals(info.getName())) {
			           UIManager.setLookAndFeel(info.getClassName());
			           break;
			       }
		 }
		} catch (Exception e) {
			System.out.print(e.getMessage());
		}

		setTitle("Java Concordance");
		setResizable(false);

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 768, 325);
		this.contentPane = new JPanel();
		this.contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(this.contentPane);
		this.contentPane.setLayout(null);

		JMenuBar menuBar = new JMenuBar();
		menuBar.setBounds(0, 0, 768, 28);
		this.contentPane.add(menuBar);

		JMenu mnOptions = new JMenu("Options");
		menuBar.add(mnOptions);

		JMenuItem mntmExit = new JMenuItem("Exit");
		mntmExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				
				System.exit(DISPOSE_ON_CLOSE);
			}
		});
		mnOptions.add(mntmExit);

		JMenu mnAbout = new JMenu("About");
		menuBar.add(mnAbout);

		JMenuItem mntmAbout = new JMenuItem("About");
		mntmAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				
				JOptionPane.showMessageDialog(contentPane, "Java Concordance \n Jonathan Field \n jof2@aber.ac.uk");
			}
		});
		mnAbout.add(mntmAbout);

		this.panel = new JPanel();
		this.panel
				.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		this.panel.setBounds(10, 34, 752, 265);
		this.contentPane.add(this.panel);
		this.panel.setLayout(null);

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(363, 133, 381, 127);
		this.panel.add(scrollPane);

		this.txtFieldContext = new JTextArea();
		scrollPane.setViewportView(this.txtFieldContext);
		this.txtFieldContext.setFont(new Font("Lucida Grande", Font.PLAIN, 17));
		this.txtFieldContext.setLineWrap(true);

		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(180, 0, 179, 260);
		this.panel.add(scrollPane_1);

		this.lineNumberList = new JList();
		this.lineNumberList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.lineNumberList.setModel(new CustomModel(this.fileReader));
		scrollPane_1.setViewportView(this.lineNumberList);
		this.lineNumberList
				.addListSelectionListener(new ListSelectionListener() {

					@Override
					public void valueChanged(ListSelectionEvent e) {

						ConcordanceFrame.this.customLineNumberModel = new CustomModelLineNumbers(ConcordanceFrame.this.fileReader,ConcordanceFrame.this.lineNumberList.getSelectedValue().toString());

						ConcordanceFrame.this.txtFieldSelectedWord.setText(ConcordanceFrame.this.indexWordList.getSelectedValue().toString());

						updateContext();
					}

				});

		this.txtFieldSelectedWord = new JTextField();
		this.txtFieldSelectedWord.setFont(new Font("Lucida Grande", Font.BOLD | Font.ITALIC, 30));
		this.txtFieldSelectedWord.setHorizontalAlignment(SwingConstants.CENTER);
		this.txtFieldSelectedWord.setBounds(363, 0, 264, 130);
		this.panel.add(this.txtFieldSelectedWord);
		this.txtFieldSelectedWord.setColumns(10);

		JButton btnLoadIndex = new JButton("Load Index");
		btnLoadIndex.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				JFileChooser fileFinder = new JFileChooser();
				fileFinder.setDialogTitle("Please choose Index File");

				fileFinder.showOpenDialog(ConcordanceFrame.this.contentPane);

				ConcordanceFrame.this.indexToLoad = fileFinder.getSelectedFile().toString();

				

			}
		});
		btnLoadIndex.setBounds(629, 19, 117, 29);
		this.panel.add(btnLoadIndex);

		JButton btnLoadText = new JButton("Load Text");
		btnLoadText.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				JFileChooser fileFinder = new JFileChooser();
				fileFinder.setDialogTitle("Please choose Text File");

				fileFinder.showOpenDialog(ConcordanceFrame.this.contentPane);

				ConcordanceFrame.this.textToLoad = fileFinder.getSelectedFile().toString();

			

			}
		});
		btnLoadText.setBounds(629, 60, 117, 29);
		this.panel.add(btnLoadText);

		JButton btnGenerate = new JButton("Generate");
		btnGenerate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {

				ConcordanceFrame.this.fileReader.processInputIndex(
						ConcordanceFrame.this.indexToLoad,
						ConcordanceFrame.this.textToLoad);

				updateList();
				updateLineNumbers();

			}
		});
		btnGenerate.setBounds(629, 101, 117, 29);
		this.panel.add(btnGenerate);

		JScrollPane scrollPane_2 = new JScrollPane();
		scrollPane_2.setBounds(0, 0, 178, 260);
		this.panel.add(scrollPane_2);

		this.indexWordList = new JList();
		
		this.indexWordList.addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent e) {



				updateLineNumbers();

			}

		});
		this.indexWordList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.indexWordList.setModel(new CustomModel(this.fileReader));

		scrollPane_2.setViewportView(this.indexWordList);

	}

	/**
	 * This method is called to dynamically update the first JList with index words,
	 * A custom table model is called to do this
	 */
	public void updateList() {

		this.indexWordList.setModel(new CustomModel(this.fileReader));
		// indexWordList.setModel(listModel);

	}

	
	/**
	 * This method is called to dynamically update the second JList with line numbers that correspond to the index words in JList one
	 * A custom table model is called to do this
	 */
	public void updateLineNumbers() {

		String selected = (String) this.indexWordList.getSelectedValue();
		this.lineNumberList.setModel(new CustomModelLineNumbers(this.fileReader,
				selected));

	}
	
	
	
	/**
	 * This method provides the user with the context of their chosen index word,
	 * the line number and selected index word are used to return the context.
	 * The context is then shown in a textField.
	 */
	public void updateContext() {
		String selected = (String) this.indexWordList.getSelectedValue();
		int selectedLine = this.lineNumberList.getSelectedIndex();

		String context = this.fileReader.getResultsList(selected).get(selectedLine)
				.getContext();

		this.txtFieldContext.setText("\n");
		this.txtFieldContext.append("\n");
		this.txtFieldContext.append(context);
		this.txtFieldContext.append("\n");

	}
	

}
