package Tests;

import static org.junit.Assert.*;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.LinkedList;

import org.junit.Before;
import org.junit.Test;

import Concordance.ConcordanceResult;
import Concordance.FileInputReader;

/**
 * 
 * @author Jonathan Field jof2@aber.ac.uk 073717922
 * @version 1.00
 *
 *The FileInputReaderTest class is designed to test the functionality of methods and output in the FileInputReader class.
 *Static Variables are defined to prevent any mutation to the variable.
 */
public class FileInputReaderTest {

	private FileInputReader fir;
	private static final String INDEX_FILE_PATH = "/Users/Jon/Dropbox/indextest.txt";
	private static final String TEXT_FILE_PATH = "/Users/Jon/Dropbox/olivertwisttest.txt";
	private static final String INDEXSORT_FILE_PATH = "/Users/Jon/Dropbox/indexsort.txt";
	private static final String FIRSTINDEXWORD = "oliver";
	private static final String SECONDINDEXWORD = "twist";
	private static final String OLIVERCONTEXT = "case it is Oliver more than probable that these memoirs would never";
	private static final int OLIVER_LINENUMBER = 4;
	private static final int TWIST_LINENUMBER = 6;
	private Hashtable<String, LinkedList<ConcordanceResult>> resultHashTable;
	private LinkedList<ConcordanceResult> resultLinkedListOne;
	private LinkedList<ConcordanceResult> resultLinkedListTwo;


	
	@Before
	public void setUp() throws Exception {
		
		this.fir = new FileInputReader();
		
	}

	/**
	 * This test ensures that the default line number is being set on Constructor call to ensure that the correct line numbers are produced for the Concordance.
	 */
	@Test
	public void testLineNumberDefaultSet() {
		
		int expectedResult = 1;
		int actualResult = fir.getLineNumber();
		
			assertEquals("Line Number not set to default",expectedResult,actualResult);
		
	}
	
	/**
	 * The purpose of this test is to ensure that index words are being succesfully read in from a textfile and store in the HashTable.
	 * 
	 * The processInputIndex method is called to read in files via a FileInputStream and a BufferedReader.
	 * The index words are then added to the HashTable with an empty line number.
	 * The HashTable is then returned and the index words are checked against the HashTable to ensure they contain the key.
	 * @throws FileNotFoundException
	 */
	@Test
	public void testInputIndex() throws FileNotFoundException{
		
		
		this.fir.processInputIndex(INDEX_FILE_PATH, TEXT_FILE_PATH);
		
			resultHashTable = this.fir.returnHashTable();
		
		
		
			assertTrue(resultHashTable.containsKey(FIRSTINDEXWORD));
		
				assertTrue(resultHashTable.containsKey(SECONDINDEXWORD));
		
					assertFalse("Word was found index",resultHashTable.containsKey("WordDoesntExistInIndex"));
		
		
		
	}
	
	/**
	 * This test ensures that the concordance is being generated correctly, The index file is first read in to the HashTable with default linenumbers.
	 * The index words are then checked to see if they are in the reference text, if they are in the text they are added to the HashTable with a line number and context.
	 * This test ensures that it is working correctly but checking that the index words are matched correctly the line number that they appear in the text.
	 */
	@Test
	public void testInputTextConcordance(){
		
		this.fir.processInputIndex(INDEX_FILE_PATH, TEXT_FILE_PATH);
		
			resultHashTable = this.fir.returnHashTable();
			
					resultLinkedListOne = resultHashTable.get(FIRSTINDEXWORD);
				
					resultLinkedListTwo = resultHashTable.get(SECONDINDEXWORD);
					
				
				ConcordanceResult resultOne =	resultLinkedListOne.get(0);
					
				ConcordanceResult resultTwo =	resultLinkedListTwo.get(0);
				
		
						int actualOliverLineNumber = resultOne.getLineNumber();
						
						int actualTwistLineNumber = resultTwo.getLineNumber();
						
						
								assertEquals("Oliver was not found at the correct lineNumber",OLIVER_LINENUMBER,actualOliverLineNumber);
			 
								assertEquals("Twist was not found at the correct lineNumber",TWIST_LINENUMBER,actualTwistLineNumber);
			 
	}
	
	
	
	/**
	 * This test ensures that the contextual information is being stored correctly.
	 * The index and text files are processed and then the HashTable is returned, the index word is used to retrieve the LinkedList structure that stores both the line number and context,
	 * The getter method is then called to retrieve the context, the retrieved context is then checked against a static variable containing the actual context to ensure that they match.
	 */
	@Test
	public void testContext(){
		
		this.fir.processInputIndex(INDEX_FILE_PATH, TEXT_FILE_PATH);
		
		resultHashTable = this.fir.returnHashTable();
		
				resultLinkedListOne = resultHashTable.get(FIRSTINDEXWORD);
				
					ConcordanceResult resultOne =	resultLinkedListOne.get(0);
					
					String actualContext = resultOne.getContext();
					
						assertEquals("Context did not match", OLIVERCONTEXT, actualContext);
		
		
	}
	
	
	/**
	 * This method ensures that the correct HashTable is being returned to ensure that there are no errors when retrieving values from the HashTable.
	 * Two separate FileInputReader objects are instantiated and then processed and returned.
	 * The two HashTables are checked to ensure they both contain the same data.
	 */
	@Test
	public void testReturnedHashTableEqualsActual(){
		
		fir.processInputIndex(INDEX_FILE_PATH, TEXT_FILE_PATH);
		resultHashTable = fir.returnHashTable();
		
		FileInputReader fileInput = new FileInputReader();
		
		fileInput.processInputIndex(INDEX_FILE_PATH, TEXT_FILE_PATH);
		
		Hashtable<String,LinkedList<ConcordanceResult>> returnedHashTable = fir.returnHashTable();
		
		assertEquals("HashTables not Equal",returnedHashTable,resultHashTable);
		
		
	}
	
	/**
	 * This method checks to ensure that the Index words are being sorted correctly in alphabetical order.
	 * An index file is read in and then hashed in the HashTable using MD5, the contents of that HashTable are then added to an ArrayList.
	 * The contexts of the text file are "apple" , "banana", "orange" and "pineapple" in non alphabetical order.
	 * The list is then sorted alphabetically and compared against an alphabetically sorted version of the List to ensure that they match, 
	 * thus ensuring that the sort is working correctly.
	 */
	@Test
	public void testSortingCorrectly(){
		
		fir.processInputIndex(INDEXSORT_FILE_PATH, TEXT_FILE_PATH);
		
		ArrayList<String> sortedList = fir.returnList();
		
		ArrayList<String> expected = new ArrayList<String>();
		expected.add("apple");
		expected.add("banana");
		expected.add("orange");
		expected.add("pineapple");
		
		
		assertEquals("List not sorted",expected,sortedList);
		
		
	}
	

}
